import { TextEditor } from './text_editor';
import AddText from './add_text.js';

export class ContentContainer {
  constructor(options) {
    this.pageViewer = options.pageViewer;
    this.eventBus = options.eventBus;
    this.pageIndex = options.pageIndex;
    this.doc = options.doc;
    this.messageHandler = options.messageHandler;
    this.pageDiv = options.pageDiv;
    this.viewport = options.viewport;
    this.scale = options.scale;

    this.destroyed = false;
    this.frameEditorList = [];
    this.tool = this.pageViewer.tool || ''
    this.color = this.pageViewer.color || ''
    this._selectedFrameIndex = -1

    this.onHandleTool = this.handleTool.bind(this)
  }

  async init () {
    const { editPagePtr, pagePtr } = await this.messageHandler.sendWithPromise('InitEditPage', {
      doc: this.doc,
      pageIndex: this.pageIndex
    })
    this.editPagePtr = editPagePtr
    this.pagePtr = pagePtr
  }

  async render () {
    await this.init()

    if (this.contentContainer) {
      this.contentContainer.style.display = 'block'
      return
    }
    
    this.eventBus._on('toolChanged', this.onHandleTool)

    const contentContainer = document.createElement('div')
    contentContainer.className = 'contentContainer'
    this.contentContainer = contentContainer
    this.pageDiv.append(this.contentContainer)

    await this.messageHandler.send('BeginEdit', {
      editPagePtr: this.editPagePtr,
      type: 1
    })

    let editAreaCount = await this.messageHandler.sendWithPromise('GetEditAreaCount', this.editPagePtr)
    
    for (let i = 0; i < editAreaCount; i++) {
      let editAreaPtr = await this.messageHandler.sendWithPromise('GetEditArea', {
        editPagePtr: this.editPagePtr,
        index: i
      })

      if (editAreaPtr) {
        let isTextPtr = await this.messageHandler.sendWithPromise('IsTextArea', editAreaPtr)
        if (isTextPtr) {
          let frameEditor = new TextEditor({
            eventBus: this.eventBus,
            contentContainer: this,
            container: this.contentContainer,
            pagePtr: this.pagePtr,
            editPagePtr: this.editPagePtr,
            editAreaPtr,
            editAreaIndex: i,
            viewport: this.viewport,
            scale: this.scale,
            messageHandler: this.messageHandler,
            pageViewer: this.pageViewer
          })
          this.frameEditorList.push(frameEditor)
        }
      }
    }

    this.destroyed = false;
  }

  destroy () {
    if (this.contentContainer && this.contentContainer.style.display !== 'none') {
      this.contentContainer.style.display = 'none'
    }
    // console.log(this.frameEditorList)

    if (this.textManager) {
      this.textManager.reset()
    }

    this.destroyed = true;

    this.eventBus._off('toolChanged', this.onHandleTool)
  }

  handleTool({
    tool,
    color
  }) {
    this.tool = tool
    this.color = color

    if (tool === 'addText') {
      this.textManager = new AddText({
        tool,
        color,
        container: this.pageDiv,
        viewport: this.viewport,
        scale: this.scale,
        page: this.page,
        eventBus: this.eventBus,
        contentContainer: this
      })
    }
    if (this.textManager) {
      this.textManager.tool = tool
      this.textManager.color = color
    }
  }

  // 添加编辑区域
  async addTextEditor (data) {
    const { rect, fontData, alignType } = data
    const param = {
      pagePtr: this.pagePtr,
      editPagePtr: this.editPagePtr,
      rect,
      fontData,
      alignType
    }
    const editAreaPtr = await this.messageHandler.sendWithPromise('CreateNewTextArea', param)

    let frameEditor = new TextEditor({
      eventBus: this.eventBus,
      contentContainer: this,
      container: this.contentContainer,
      pagePtr: this.pagePtr,
      editPagePtr: this.editPagePtr,
      editAreaPtr,
      editAreaIndex: this.frameEditorList.length,
      viewport: this.viewport,
      scale: this.scale,
      messageHandler: this.messageHandler,
      newAdd: true,
      pageViewer: this.pageViewer
    })
    this.frameEditorList.push(frameEditor)
  }

  // 更新大区域图片
  updateSomeCanvas(area) {
    this.frameEditorList.forEach(item => {
      const frame = item.frame
      if (frame.right < area.left || frame.left > area.right || frame.bottom < area.top || frame.top > area.bottom) {
        // console.log(item.editAreaIndex, '没重叠')
      } else {
        // console.log(item.editAreaIndex, '重叠了！！')
        item.updateCanvas()
      }
    })
  }

  get selectedFrameIndex () {
    return this._selectedFrameIndex
  }

  set selectedFrameIndex (index) {
    this._selectedFrameIndex = index
  }

  removeEditor (index) {
    this.frameEditorList.splice(index, 1)
  }
}
